@@ -43,7 +43,7 @@ int wait_timer (struct thread *);
 int backup_seen (struct thread *);
 int neighbor_change (struct thread *);
 
-int dr_change (struct ospf6_if *);
+int dr_change (struct ospf6_interface *);
 
 #endif /* OSPF6_ISM_H */
 
