@@ -1,6 +1,6 @@
 /*
  * OSPF inter-area routing.
- * Copyright (C) 1999 Alex Zinin
+ * Copyright (C) 1999, 2000 Alex Zinin, Toshiaki Takada
  *
  * This file is part of GNU Zebra.
  *
@@ -17,7 +17,8 @@
  * You should have received a copy of the GNU General Public License
  * along with GNU Zebra; see the file COPYING.  If not, write to the Free
  * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.  */
+ * 02111-1307, USA.
+ */
 
 
 #include <zebra.h>
@@ -33,9 +34,9 @@
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_interface.h"
 #include "ospfd/ospf_ism.h"
+#include "ospfd/ospf_lsa.h"
 #include "ospfd/ospf_neighbor.h"
 #include "ospfd/ospf_nsm.h"
-#include "ospfd/ospf_lsa.h"
 #include "ospfd/ospf_spf.h"
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_ase.h"
@@ -47,8 +48,8 @@
 
 struct ospf_route *
 ospf_find_abr_route (struct route_table *rtrs, 
-		     struct prefix_ipv4 *abr,
-		     struct ospf_area *area)
+                     struct prefix_ipv4 *abr,
+                     struct ospf_area *area)
 {
   struct route_node *rn;
   struct ospf_route *or;
@@ -61,7 +62,7 @@ ospf_find_abr_route (struct route_table *rtrs,
 
   for (node = listhead ((list) rn->info); node; nextnode (node))
     if ((or = getdata (node)) != NULL)
-      if (or->area == area && (or->flags & ROUTER_LSA_BORDER))
+      if (or->u.std.area == area && (or->u.std.flags & ROUTER_LSA_BORDER))
 	return or;
 
   return NULL;
@@ -69,13 +70,13 @@ ospf_find_abr_route (struct route_table *rtrs,
 
 void
 ospf_ia_network_route (struct route_table *rt, struct prefix_ipv4 *p,
-		       struct ospf_route *new_or, struct ospf_route *abr_or)
+                       struct ospf_route *new_or, struct ospf_route *abr_or)
 {
   struct route_node *rn1;
   struct ospf_route *or;
 
   zlog_info ("Z: ospf_ia_network_route(): processing summary route to %s/%d", 
-	     inet_ntoa (p->prefix), p->prefixlen);
+             inet_ntoa (p->prefix), p->prefixlen);
 
   /* Find a route to the same dest */
   if ((rn1 = route_node_lookup (rt, (struct prefix *) p)))
@@ -89,33 +90,26 @@ ospf_ia_network_route (struct route_table *rt, struct prefix_ipv4 *p,
 	  zlog_info ("Z: ospf_ia_network_route(): "
 		     "Found a route to the same network");
 
-	  /*Check the existing route*/
-	  res = ospf_route_cmp (new_or, or);
-
-	  switch (res)
+	  /* Check the existing route. */
+	  if ((res = ospf_route_cmp (new_or, or)) < 0)
+	    ospf_route_subst (rn1, new_or, abr_or);
+	  else if (res == 0)
 	    {
-	    case 1:
-	      ospf_route_subst (rn1, new_or, abr_or);
-	      break;
-
-	    case -1:
 	      ospf_route_free (new_or);
 	      return; /* Sorry, you're worse*/
-
-	    case 0:
+	    }
+	  else
+	    {
 	      route_lock_node (rn1);
 	      ospf_route_copy_nexthops (or, abr_or->path);
 	      route_unlock_node (rn1);
-
-	    default:
-	      break;
-	    }; /* switch() */
+	    }
 	} /* if (or)*/
     } /*if (rn1)*/
   else
     { /* no route */
       zlog_info ("Z: ospf_ia_network_route(): add new route to %s/%d",
-		 inet_ntoa (p->prefix), p->prefixlen);
+                 inet_ntoa (p->prefix), p->prefixlen);
 
       ospf_route_add (rt, p, new_or, abr_or);
     }
@@ -124,14 +118,14 @@ ospf_ia_network_route (struct route_table *rt, struct prefix_ipv4 *p,
 
 void
 ospf_ia_router_route (struct route_table *rt, struct prefix_ipv4 *p,
-		      struct ospf_route *new_or, struct ospf_route *abr_or)
+                      struct ospf_route *new_or, struct ospf_route *abr_or)
 {
   struct route_node *rn;
   struct ospf_route *or = NULL;
   int ret;
 
   zlog_info ("Z: ospf_ia_router_route(): considering %s/%d", 
-	     inet_ntoa (p->prefix), p->prefixlen);
+             inet_ntoa (p->prefix), p->prefixlen);
 
   /* Find a route to the same dest */
   rn = route_node_get (rt,(struct prefix *) p);
@@ -143,36 +137,38 @@ ospf_ia_router_route (struct route_table *rt, struct prefix_ipv4 *p,
     {
       /* This is an additional route */
       route_unlock_node (rn);
-      or = ospf_find_asbr_route_through_area (rt, p, new_or->area);
+      or = ospf_find_asbr_route_through_area (rt, p, new_or->u.std.area);
     }
 
   if (or)
     {
       zlog_info ("Z: ospf_ia_router_route(): "
-		 "a route to the same ABR through the same area exists");
+                 "a route to the same ABR through the same area exists");
 
-    ret = ospf_route_cmp (new_or, or);
-    if (ret == 1)   /* New route is better */
-      {
-	list_delete_by_val (rn->info, or);
-	ospf_route_free (or);
-	/* proceed down */
-      }
-    else if (ret == 0)  /* Routes are the same */
-      {
-	zlog_info ("Z: ospf_ia_router_route(): merging the new route");
+      /* New route is better */
+      if ((ret = ospf_route_cmp (new_or, or)) < 0)
+	{
+	  list_delete_by_val (rn->info, or);
+	  ospf_route_free (or);
+	  /* proceed down */
+	}
+      /* Routes are the same */
+      else if (ret == 0)
+	{
+	  zlog_info ("Z: ospf_ia_router_route(): merging the new route");
 
-	ospf_route_copy_nexthops (or, abr_or->path);
-	ospf_route_free (new_or);
-	return;
-      }
-    else   /* New route is worse */
-      {
-	zlog_info ("Z: ospf_ia_router_route(): skipping the new route");
+	  ospf_route_copy_nexthops (or, abr_or->path);
+	  ospf_route_free (new_or);
+	  return;
+	}
+      /* New route is worse */
+      else
+	{
+	  zlog_info ("Z: ospf_ia_router_route(): skipping the new route");
 
-	ospf_route_free (new_or);
-	return;
-      }
+	  ospf_route_free (new_or);
+	  return;
+	}
     }
 
   ospf_route_copy_nexthops (new_or, abr_or->path);
@@ -253,10 +249,10 @@ process_summary_lsa (struct ospf_lsa *l, void *v, int i)
   new_or->type = OSPF_DESTINATION_NETWORK;
   new_or->id = sl->header.id;
   new_or->mask = sl->mask;
-  new_or->options = sl->header.options;
-  new_or->origin = (struct lsa_header *) sl;
+  new_or->u.std.options = sl->header.options;
+  new_or->u.std.origin = (struct lsa_header *) sl;
   new_or->cost = abr_or->cost + metric;
-  new_or->area = args->area;
+  new_or->u.std.area = args->area;
   new_or->path_type = OSPF_PATH_INTER_AREA;
 
   if (sl->header.type == OSPF_SUMMARY_LSA)
@@ -264,7 +260,7 @@ process_summary_lsa (struct ospf_lsa *l, void *v, int i)
   else 
     {
       new_or->type = OSPF_DESTINATION_ROUTER;
-      new_or->flags = ROUTER_LSA_EXTERNAL;
+      new_or->u.std.flags = ROUTER_LSA_EXTERNAL;
       ospf_ia_router_route (args->rtrs, &p, new_or, abr_or);
     }
 
@@ -273,9 +269,9 @@ process_summary_lsa (struct ospf_lsa *l, void *v, int i)
 
 void
 ospf_examine_summaries (struct ospf_area * area,
-			struct ospf_lsdb *lsdb,
-			struct route_table *rt,
-			struct route_table *rtrs)
+                        struct ospf_lsdb *lsdb,
+                        struct route_table *rt,
+                        struct route_table *rtrs)
 {
   struct ia_args args = {rt, rtrs, area};
   ospf_lsdb_iterator (lsdb, &args, 0, process_summary_lsa);
@@ -291,9 +287,9 @@ ospf_area_is_transit (struct ospf_area *area)
 void
 ospf_update_network_route (struct route_table *rt, 
                            struct route_table *rtrs,
-			   struct summary_lsa *lsa,
-			   struct prefix_ipv4 *p,
-			   struct ospf_area *area)
+                           struct summary_lsa *lsa,
+                           struct prefix_ipv4 *p,
+                           struct ospf_area *area)
 {
   struct route_node *rn;
   struct ospf_route *or, *abr_or, *new_or;
@@ -321,19 +317,19 @@ ospf_update_network_route (struct route_table *rt,
     {
       if (ospf_top->abr_type != OSPF_ABR_SHORTCUT)
         return; /* Standard ABR can update only already installed
-		   backbone paths					*/
+                   backbone paths                                       */
 
       zlog_info ("Z: ospf_update_network_route(): "
-		 "Allowing Shortcut ABR to add new route");
+                 "Allowing Shortcut ABR to add new route");
 
       new_or = ospf_route_new ();
       new_or->type = OSPF_DESTINATION_NETWORK;
       new_or->id = lsa->header.id;
       new_or->mask = lsa->mask;
-      new_or->options = lsa->header.options;
-      new_or->origin = (struct lsa_header *) lsa;
+      new_or->u.std.options = lsa->header.options;
+      new_or->u.std.origin = (struct lsa_header *) lsa;
       new_or->cost = cost;
-      new_or->area = area;
+      new_or->u.std.area = area;
       new_or->path_type = OSPF_PATH_INTER_AREA;
       ospf_route_add (rt, p, new_or, abr_or);
 
@@ -343,7 +339,7 @@ ospf_update_network_route (struct route_table *rt,
     {
       route_unlock_node (rn);
       if (rn->info == NULL)
-	return;
+        return;
     }
 
   or = rn->info;
@@ -358,7 +354,7 @@ ospf_update_network_route (struct route_table *rt,
   if (ospf_top->abr_type == OSPF_ABR_SHORTCUT)
     {
       if (or->path_type == OSPF_PATH_INTRA_AREA &&
-	  or->area->area_id.s_addr != OSPF_AREA_BACKBONE)
+	  !OSPF_IS_AREA_BACKBONE (or->u.std.area))
 	{
 	  zlog_info ("Z: ospf_update_network_route(): Shortcut: "
 		     "this intra-area path is not backbone");
@@ -367,7 +363,7 @@ ospf_update_network_route (struct route_table *rt,
     }
   else   /* Not Shortcut ABR */
     {
-      if (or->area->area_id.s_addr != OSPF_AREA_BACKBONE)
+      if (!OSPF_IS_AREA_BACKBONE (or->u.std.area))
 	{
 	  zlog_info ("Z: ospf_update_network_route(): "
 		     "route is not BB-associated");
@@ -375,7 +371,7 @@ ospf_update_network_route (struct route_table *rt,
 	}
     }
 
-  if (or->cost <  cost)
+  if (or->cost < cost)
     {
       zlog_info ("Z: ospf_update_network_route(): new route is worse");
       return;
@@ -384,36 +380,36 @@ ospf_update_network_route (struct route_table *rt,
   if (or->cost == cost)
     {
       zlog_info ("Z: ospf_update_network_route(): "
-		 "new route is same distance, adding nexthops");
+                 "new route is same distance, adding nexthops");
       ospf_route_copy_nexthops (or, abr_or->path);
     }
 
-  if (or->cost >  cost)
+  if (or->cost > cost)
     {
       zlog_info ("Z: ospf_update_network_route(): "
-		 "new route is better, overriding nexthops");
+                 "new route is better, overriding nexthops");
       ospf_route_subst_nexthops (or, abr_or->path);
       or->cost = cost;
 
       if ((ospf_top->abr_type == OSPF_ABR_SHORTCUT) &&
-	  (or->area->area_id.s_addr != OSPF_AREA_BACKBONE))
+	  !OSPF_IS_AREA_BACKBONE (or->u.std.area))
 	{
 	  or->path_type = OSPF_PATH_INTER_AREA;
-	  or->area = area;
+	  or->u.std.area = area;
 
           /* Note that we can do this only in Shortcut ABR mode,
-	     because standard ABR must leave the route type and area
-	     unchanged
-	  */
-	}
+             because standard ABR must leave the route type and area
+             unchanged
+          */
+        }
     }
 }
 
 void
 ospf_update_router_route (struct route_table *rtrs, 
-			  struct summary_lsa *lsa,
-			  struct prefix_ipv4 *p,
-			  struct ospf_area *area)
+                          struct summary_lsa *lsa,
+                          struct prefix_ipv4 *p,
+                          struct ospf_area *area)
 {
   struct ospf_route *or, *abr_or, *new_or;
   struct prefix_ipv4 abr;
@@ -452,7 +448,7 @@ ospf_update_router_route (struct route_table *rtrs,
          /* route to ASBR through the BB not found
             the router is not Shortcut ABR, exiting */
 
-	  return;
+          return;
       else
 	/* We're a Shortcut ABR*/
 	{
@@ -463,21 +459,21 @@ ospf_update_router_route (struct route_table *rtrs,
 	  new_or->type = OSPF_DESTINATION_ROUTER;
 	  new_or->id = lsa->header.id;
 	  new_or->mask = lsa->mask;
-	  new_or->options = lsa->header.options;
-	  new_or->origin = (struct lsa_header *)lsa;
+	  new_or->u.std.options = lsa->header.options;
+	  new_or->u.std.origin = (struct lsa_header *)lsa;
 	  new_or->cost = cost;
-	  new_or->area = area;
+	  new_or->u.std.area = area;
 	  new_or->path_type = OSPF_PATH_INTER_AREA;
-	  new_or->flags = ROUTER_LSA_EXTERNAL;
+	  new_or->u.std.flags = ROUTER_LSA_EXTERNAL;
 	  ospf_ia_router_route (rtrs, p, new_or, abr_or);
 
-	  return;
-	}
+          return;
+        }
     }
 
   /* At this point the "or" is always bb-associated */
 
-  if (!(or->flags & ROUTER_LSA_EXTERNAL))
+  if (!(or->u.std.flags & ROUTER_LSA_EXTERNAL))
     {
       zlog_info ("Z: ospf_upd_router_route(): the remote router is not an ASBR");
       return;
@@ -487,20 +483,20 @@ ospf_update_router_route (struct route_table *rtrs,
       or->path_type != OSPF_PATH_INTER_AREA)
     return;
 
-  if (or->cost <  cost)
+  if (or->cost < cost)
     return;
 
   else if (or->cost == cost)
     ospf_route_copy_nexthops (or, abr_or->path);
 
-  else if (or->cost >  cost)
+  else if (or->cost > cost)
     {
       ospf_route_subst_nexthops (or, abr_or->path);
       or->cost = cost;
 
       /* Even if the ABR runs in Shortcut mode, we can't change
-	 the path type and area, because the "or" is always bb-associated
-	 at this point and even Shortcut ABR can't change these attributes */
+         the path type and area, because the "or" is always bb-associated
+         at this point and even Shortcut ABR can't change these attributes */
     }
 }
 
@@ -519,11 +515,10 @@ process_transit_summary_lsa (struct ospf_lsa *l, void *v, int i)
   sl = (struct summary_lsa *) l->data;
 
   zlog_info ("Z: process_transit_summaries(): LS ID: %s",
-	     inet_ntoa (l->data->id));
+             inet_ntoa (l->data->id));
 
   metric = GET_METRIC (sl->metric);
    
-
   if (metric == OSPF_LS_INFINITY)
     {
       zlog_info ("Z: process_transit_summaries(): metric is infinity, skip");
@@ -562,9 +557,9 @@ process_transit_summary_lsa (struct ospf_lsa *l, void *v, int i)
 
 void
 ospf_examine_transit_summaries (struct ospf_area *area,
-			        struct ospf_lsdb *lsdb,
-			        struct route_table *rt,
-			        struct route_table *rtrs)
+                                struct ospf_lsdb *lsdb,
+                                struct route_table *rt,
+                                struct route_table *rtrs)
 {
   struct ia_args args = {rt, rtrs, area};
 
@@ -573,7 +568,7 @@ ospf_examine_transit_summaries (struct ospf_area *area,
 
 void
 ospf_ia_routing (struct route_table *rt,
-		 struct route_table *rtrs)
+                 struct route_table *rtrs)
 {
   struct ospf_area * area;
 
@@ -585,81 +580,87 @@ ospf_ia_routing (struct route_table *rt,
       struct ospf_area *area;
 
       switch (ospf_top->abr_type)
-	{
-	case OSPF_ABR_STAND:
-	  zlog_info ("Z: ospf_ia_routing():Standard ABR");
+        {
+        case OSPF_ABR_STAND:
+          zlog_info ("Z: ospf_ia_routing():Standard ABR");
 
-	  if ((area = ospf_top->backbone))
-	    {
-	      listnode node;
+          if ((area = ospf_top->backbone))
+            {
+              listnode node;
 
-	      zlog_info ("Z: ospf_ia_routing():backbone area found");
-	      zlog_info ("Z: ospf_ia_routing():examining summaries");
-	      OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
+              zlog_info ("Z: ospf_ia_routing():backbone area found");
+              zlog_info ("Z: ospf_ia_routing():examining summaries");
+              OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
 
-	      LIST_ITERATOR (ospf_top->areas, node)
-		if ((area = getdata (node)) != NULL)
+              LIST_ITERATOR (ospf_top->areas, node)
+                if ((area = getdata (node)) != NULL)
                   if (area != ospf_top->backbone)
-		  if (ospf_area_is_transit (area))
-		    OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL (area, rt, rtrs);
-	    }
-	  else
-	    zlog_info ("Z: ospf_ia_routing():backbone area NOT found");
-	  break;
-	case OSPF_ABR_IBM:
-	case OSPF_ABR_CISCO:
-	  zlog_info ("Z: ospf_ia_routing():Alternative Cisco/IBM ABR");
+                  if (ospf_area_is_transit (area))
+                    OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL (area, rt, rtrs);
+            }
+          else
+            zlog_info ("Z: ospf_ia_routing():backbone area NOT found");
+          break;
+        case OSPF_ABR_IBM:
+        case OSPF_ABR_CISCO:
+          zlog_info ("Z: ospf_ia_routing():Alternative Cisco/IBM ABR");
 
-	  area = ospf_top->backbone; /* Find the BB */
+          area = ospf_top->backbone; /* Find the BB */
 
-	  /* If we have an active BB connection */
-	  if (area && ospf_act_bb_connection ())
-	    {
-	      zlog_info ("Z: ospf_ia_routing(): backbone area found");
-	      zlog_info ("Z: ospf_ia_routing(): examining BB summaries");
+          /* If we have an active BB connection */
+          if (area && ospf_act_bb_connection ())
+            {
+              zlog_info ("Z: ospf_ia_routing(): backbone area found");
+              zlog_info ("Z: ospf_ia_routing(): examining BB summaries");
 
-	      OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
+              OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
 
-	      LIST_ITERATOR (ospf_top->areas, node)
-		if ((area = getdata (node)) != NULL)
+              LIST_ITERATOR (ospf_top->areas, node)
+                if ((area = getdata (node)) != NULL)
                   if (area != ospf_top->backbone)
-		  if (ospf_area_is_transit (area))
-		    OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL (area, rt, rtrs);
-	    }
-	  else
-	    { /* No active BB connection--consider all areas */
-	      zlog_info ("Z: ospf_ia_routing(): "
-			 "Active BB connection not found");
+                  if (ospf_area_is_transit (area))
+                    OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL (area, rt, rtrs);
+            }
+          else
+            { /* No active BB connection--consider all areas */
+              zlog_info ("Z: ospf_ia_routing(): "
+                         "Active BB connection not found");
 
-	      LIST_ITERATOR (ospf_top->areas, node)
-		if ((area = getdata (node)) != NULL)
-		  OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
-	    }
-	  break;
-	case OSPF_ABR_SHORTCUT:
-	  zlog_info ("Z: ospf_ia_routing():Alternative Shortcut");
+              LIST_ITERATOR (ospf_top->areas, node)
+                if ((area = getdata (node)) != NULL)
+                  OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
+            }
+          break;
+        case OSPF_ABR_SHORTCUT:
+          zlog_info ("Z: ospf_ia_routing():Alternative Shortcut");
 
-	  area = ospf_top->backbone; /* Find the BB */
+          area = ospf_top->backbone; /* Find the BB */
 
-	  /* If we have an active BB connection */
-	  if (area && ospf_act_bb_connection ())
-	    {
-	      zlog_info ("Z: ospf_ia_routing(): backbone area found");
-	      zlog_info ("Z: ospf_ia_routing(): examining BB summaries");
-	      OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
-	    }
+          /* If we have an active BB connection */
+          if (area && ospf_act_bb_connection ())
+            {
+              zlog_info ("Z: ospf_ia_routing(): backbone area found");
+              zlog_info ("Z: ospf_ia_routing(): examining BB summaries");
+              OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
+            }
 
-	  LIST_ITERATOR (ospf_top->areas, node)
-	    if ((area = getdata (node)) != NULL)
+          LIST_ITERATOR (ospf_top->areas, node)
+            if ((area = getdata (node)) != NULL)
               if (area != ospf_top->backbone)
-	      if (ospf_area_is_transit (area) || 
-                  area->shortcut_capability ||
-                  (ospf_top->backbone == NULL))
-		OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL (area, rt, rtrs);
-	  break;
-	default:
-	  break;
-	}
+              if (ospf_area_is_transit (area) ||
+                  ( (area->shortcut_configured != OSPF_SHORTCUT_DISABLE) &&
+                    ( (ospf_top->backbone == NULL) ||
+                      ( (area->shortcut_configured == OSPF_SHORTCUT_ENABLE) &&
+                        area->shortcut_capability
+                      )
+                    )
+                  )
+                 )
+                OSPF_EXAMINE_TRANSIT_SUMMARIES_ALL (area, rt, rtrs);
+          break;
+        default:
+          break;
+        }
     }
   else 
     {
@@ -668,7 +669,7 @@ ospf_ia_routing (struct route_table *rt,
       zlog_info ("Z: ospf_ia_routing():not ABR, considering all areas");
 
       LIST_ITERATOR (ospf_top->areas, node)
-	if ((area = getdata (node)) != NULL)
-	  OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
+        if ((area = getdata (node)) != NULL)
+          OSPF_EXAMINE_SUMMARIES_ALL (area, rt, rtrs);
     }
 }
